# SPDX-FileCopyrightText: 2024-present Marceau <git@marceau-h.fr>
#
# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Caution: This script needs you to install the `webanno_tsv` package.
As it is not available on PyPI, we can't include it in the requirements.txt file.
You can install it by running `pip install git+https://github.com/neuged/webanno_tsv`.
"""
from dataclasses import replace
from pathlib import Path
from typing import Set, Iterable, Optional

import polars as pl
from webanno_tsv import Document, Annotation  # https://github.com/neuged/webanno_tsv

from camel2karam.converters.base_converter import BaseConverter


class Jsonl2WebAnno(BaseConverter):
    points: Set[str] = {"!", "?", "."}

    def __init__(self):
        self.layer_defs: Optional[list[tuple[str, list[str]]]] = None
        self.layers: Optional[Set[str]] = None
        self.fields_to_keep: Optional[list[str]] = None

        self.init_fields()

        super().__init__(
            file_glob="*.jsonl",
            output_suffix=".csv",
        )

    def init_fields(self, fields_dict: dict[str, list[str]] | None = None) -> None:
        if fields_dict is not None:
            assert isinstance(fields_dict, dict), f"The kw argument `fields_dict` should be a dict (or None) !"

            self.layer_def = []
            for layer, fields in fields_dict.items():
                assert isinstance(layer, str)
                assert isinstance(fields, Iterable)
                fields = list(fields) if not isinstance(fields, list) else fields

                self.layer_defs.append((layer, fields))

            self.layers = set(fields_dict)
        else:
            self.layer_defs = [("webanno.custom.KARAMLayer", ["pos"]), ]
            self.layers = {"webanno.custom.KARAMLayer", }

        self.fields_to_keep = [
            f for _, fields in self.layer_defs for f in fields
        ]

    def converter(
            self,
            input_path: str | Path,
            output_path: str | Path,
            /,
            **kwargs,
    ) -> None:
        input_path, output_path = self.converter_input_validation(input_path, output_path)

        df = pl.read_ndjson(input_path)
        words = df['word']

        points_index = [i for i, w in enumerate(words) if w.casefold().strip() in self.points]

        sentences = [
            words[:points_index[i]] if i == 0
            else words[points_index[i - 1]:points_index[i]] if i != len(points_index)
            else words[points_index[i - 1]:]
            for i in range(len(points_index) + 1)
        ]

        doc = Document.from_token_lists(sentences)

        annotations = []

        df = df.select(self.fields_to_keep)

        for i, row in enumerate(df.iter_rows(named=True)):
            for attr_field, attr_value in row.items():
                for layer, fields in self.layer_defs:
                    if attr_field in fields:
                        break
                else:
                    print(f"Skipping {attr_field}={attr_value}")
                    continue
                annotations.append(
                    Annotation(
                        tokens=doc.tokens[i:i + 1],
                        layer=layer,
                        field=attr_field,
                        label=attr_value,
                    )
                )

        doc = replace(doc, annotations=annotations, layer_defs=self.layer_defs)

        with open(output_path, "w", encoding="utf-8") as f:
            f.write(doc.tsv())


if __name__ == "__main__":
    j2w = Jsonl2WebAnno()

    j2w.folder_converter("../../output/mled_msa", "../../output/mled_msa_webanno")



# SPDX-FileCopyrightText: 2024-present Marceau <git@marceau-h.fr>
#
# SPDX-License-Identifier: AGPL-3.0-or-later
from pathlib import Path

import polars as pl

from camel2karam.converters.base_converter import BaseConverter


class CsvTok2CsvSent(BaseConverter):
    def __init__(self):
        super().__init__(
            file_glob="*.csv",
            output_suffix=".csv",
        )

    def toks_to_sents_zipped_unzipped(self, words, pos, pos2=None):
        res = self.toks_to_sents(words, pos) if pos2 is None else self.toks_to_sents(words, pos, pos2)
        return zip(*((" | ".join(e) for e in zip(*s)) for s in res))

    def converter(
            self,
            input_path: str | Path,
            output_path: str | Path,
            /,
            word_col: str = "word",
            pos_col: str = "pos",
            pos2_col: str = "xpos",
            **kwargs,
    ) -> None:
        input_path, output_path = self.converter_input_validation(input_path, output_path)

        df = pl.read_csv(input_path).fill_nan("").fill_null("")

        if pos2_col is None:
            w_col, p_col = self.toks_to_sents_zipped_unzipped(df[word_col], df[pos_col])
            new_df = pl.DataFrame({word_col: w_col, pos_col: p_col})

        else:
            w_col, p_col, p2_col = self.toks_to_sents_zipped_unzipped(df[word_col], df[pos_col], df[pos2_col])
            new_df = pl.DataFrame({word_col: w_col, pos_col: p_col, pos2_col: p2_col})

        new_df.write_csv(output_path)


if __name__ == "__main__":
    ct2cs = CsvTok2CsvSent()

    ct2cs.folder_converter(
        "../../output/mled_msa_csv",
        "../../output/mled_msa_csv_sent",
        word_col="word",
        pos_col="pos",
        pos2_col=None
    )



# SPDX-FileCopyrightText: 2024-present Marceau <git@marceau-h.fr>
#
# SPDX-License-Identifier: AGPL-3.0-or-later
from pathlib import Path

import polars as pl

from camel2karam.converters.base_converter import BaseConverter


class TagsetConvert(BaseConverter):
    TAGSET = {
        "PART_DET": "PART",
        "CONJ": "PART",
        "PREP": "PART",
        "PART_NEG": "PART",
        "PART_FUT": "PART",
        "PART_PROG": "PART",
        "CONJ_SUB": "PART",
        "PRON_DEM": "NOUN",
        "PRON_INTERROG": "NOUN",
        "PART": "PART",
        "PART_CONNECT": "PART",
        "PART_EMPHATIC": "PART",
        "PART_RC": "PART",
        "PART_VOC": "PART",
        "PRON": "NOUN",
        "NOUN": "NOUN",
        "NOUN_NUM": "NOUN",
        "NOUN_PROP": "NOUN",
        "NOUN_QUANT": "NOUN",
        "ADJ": "NOUN",
        "ADJ_NUM": "NOUN",
        "ADJ_COMP": "NOUN",
        "ADV": "NOUN",
        "ADV_REL": "NOUN",
        "VERB": "VERB",
        "VERB_PSEUDO": "VERB",
        "VERB_NOUN": "NOUN",
        "PRON_EXCLAM": "NOUN",
        "PRON_REL": "NOUN",
        "PART_VERB": "PART",
        "PART_INTERROG": "PART",
        "PART_RESTRICT": "PART",
        "PART_FOCUS": "PART",
        "DIGIT": "DIGIT",
        "ABBREV": "PART",
        "INTERJ": "PART",
        "FOREIGN": "NOUN",
        "PUNC": "PUNC"
    }

    @classmethod
    def replace(cls, tag):
        return cls.TAGSET.get(tag.upper(), cls.TAGSET.get(tag, tag.upper()))

    def __init__(self):
        super().__init__(
            file_glob="*.jsonl",
            output_suffix=".jsonl",
        )

    def converter(
            self,
            input_path: str | Path,
            output_path: str | Path,
            /,
            **kwargs,
    ) -> None:
        input_path, output_path = self.converter_input_validation(input_path, output_path)

        (
            pl
            .read_ndjson(input_path)
            .with_columns(
                [pl.col("pos").map_elements(TagsetConvert.replace, return_dtype=pl.Utf8)]
            )
            .write_ndjson(output_path)
        )


if __name__ == "__main__":
    tc = TagsetConvert()

    tc.folder_converter("../../output/mled_msa", "../../output_karamed/mled_msa_karamed")
    tc.folder_converter("../../output/mled_egy", "../../output_karamed/mled_egy_karamed")
    tc.folder_converter("../../output/bert_msa", "../../output_karamed/bert_msa_karamed")
    tc.folder_converter("../../output/bert_egy", "../../output_karamed/bert_egy_karamed")
    tc.folder_converter("../../output/bert_glf", "../../output_karamed/bert_glf_karamed")
    tc.folder_converter("../../output/bert_lev", "../../output_karamed/bert_lev_karamed")



# SPDX-FileCopyrightText: 2024-present Marceau <git@marceau-h.fr>
#
# SPDX-License-Identifier: AGPL-3.0-or-later
from pathlib import Path
from xml.sax.saxutils import escape, quoteattr

import polars as pl

from camel2karam.converters.base_converter import BaseConverter

class Jsonl2Xml(BaseConverter):
    @staticmethod
    def better_quoteattr(s):
        if isinstance(s, int):
            return f'"{s}"'
        if s is None:
            return f'"{None}"'
        return quoteattr(s)

    def __init__(self):
        super().__init__(
            file_glob="*.jsonl",
            output_suffix=".xml",
        )

    def converter(
            self,
            input_path: str | Path,
            output_path: str | Path,
            /,
            **kwargs,
    ) -> None:
        input_path, output_path = self.converter_input_validation(input_path, output_path)

        df = pl.read_ndjson(input_path)

        with output_path.open('w', encoding='utf-8') as f:
            f.write("""<?xml version="1.0" encoding="utf-8"?>\n<root>""")

            for i, row in enumerate(df.iter_rows(named=True)):
                f.write("<w ")
                f.write(" ".join(
                    f'{k}={self.better_quoteattr(v)}'
                    # if v is not None else f'{k}="{None}"'
                    for k, v in row.items()
                    if k != "word"
                ))
                f.write(f"> {escape(self.replace_rafter(row['word']))} </w>\n")

            f.write("</root>")



if __name__ == "__main__":
    j2x = Jsonl2Xml()

    j2x.folder_converter("../../output/mled_msa", "../../output/mled_msa_xml")
    j2x.folder_converter("../../output/mled_egy", "../../output/mled_egy_xml")
    j2x.folder_converter("../../output/bert_msa", "../../output/bert_msa_xml")
    j2x.folder_converter("../../output/bert_egy", "../../output/bert_egy_xml")
    j2x.folder_converter("../../output/bert_glf", "../../output/bert_glf_xml")
    j2x.folder_converter("../../output/bert_lev", "../../output/bert_lev_xml")

    j2x.folder_converter("../../output_karamed/mled_msa_karamed", "../../output_karamed/mled_msa_karamed_xml")
    j2x.folder_converter("../../output_karamed/mled_egy_karamed", "../../output_karamed/mled_egy_karamed_xml")
    j2x.folder_converter("../../output_karamed/bert_msa_karamed", "../../output_karamed/bert_msa_karamed_xml")
    j2x.folder_converter("../../output_karamed/bert_egy_karamed", "../../output_karamed/bert_egy_karamed_xml")
    j2x.folder_converter("../../output_karamed/bert_glf_karamed", "../../output_karamed/bert_glf_karamed_xml")
    j2x.folder_converter("../../output_karamed/bert_lev_karamed", "../../output_karamed/bert_lev_karamed_xml")




# SPDX-FileCopyrightText: 2024-present Marceau <git@marceau-h.fr>
#
# SPDX-License-Identifier: AGPL-3.0-or-later
from abc import ABC, abstractmethod
from dataclasses import dataclass
from pathlib import Path
from typing import Iterable, List, Tuple

from tqdm import tqdm


@dataclass
class PartsMapping:
    FORM: int = -1
    LEMMA: int = -1
    UPOS: int = -1
    XPOS: int = -1
    FEATS: int = -1
    HEAD: int = -1
    DEPREL: int = -1
    DEPS: int = -1
    MISC: int = -1


class BaseConverter(ABC):
    @abstractmethod
    def __init__(self, /, file_glob: str = "*.jsonl", output_suffix: str = ".xml") -> None:
        self.file_glob = file_glob
        self.output_suffix = output_suffix

    @staticmethod
    def replace_rafter(text: str) -> str:
        return text.replace("=\">", "=\"&gt;").replace("=\"<", "=\"&lt;")

    @staticmethod
    def toks_to_sents(*cols: Iterable[str]) -> List[List[Tuple[str, ...]]]:
        sents = []
        sent = []
        prev = None
        for i, row in enumerate(zip(*cols)):
            tok = row[0].strip()
            if len(sent) > 1 and tok not in "!؟.!?" and prev in "!؟.!?":
                sents.append(sent)
                sent = []
            sent.append(tuple(map(str.strip, row)))
            prev = tok

            # sent.append(tuple(map(str.strip, row)))
            # if tok in "!؟.!?" and sent and (len(sent) < 2 or sent[-2] not in "!؟.!?"):
            #     sents.append(sent)
            #     sent = []

        if sent:
            sents.append(sent)

        return sents

    @staticmethod
    def write_connlu(
            output_path: str | Path,
            sents: List[List[Tuple[str, ...]]],
            parts_mapping: PartsMapping,
    ) -> None:
        pm_values = list(parts_mapping.__dict__.values())
        with output_path.open("w", encoding="utf-8") as f:
            f.write("# global.columns = ID FORM LEMMA UPOS XPOS FEATS HEAD DEPREL DEPS MISC\n\n")
            for i, sent in enumerate(sents, 1):
                f.write(f"""# sent_id = {i}\n# text = {' '.join(w[parts_mapping.FORM] for w in sent)}\n""")
                for j, parts in enumerate(sent, 1):
                    to_fill = [j] + [parts[i] if i != -1 else "_" for i in pm_values]
                    f.write("\t".join(map(str, to_fill)) + "\n")
                f.write("\n")

    def converter_input_validation(
            self,
            input_path: str | Path,
            output_path: str | Path,
    ) -> tuple[Path, Path]:
        if isinstance(input_path, str):
            input_path = Path(input_path)
        elif isinstance(input_path, Path):
            pass
        else:
            raise ValueError(f"input_path must be a string or a Path object, not {type(input_path)}")

        if isinstance(output_path, str):
            output_path = Path(output_path)
        elif isinstance(output_path, Path):
            pass
        else:
            raise ValueError(f"output_path must be a string or a Path object, not {type(output_path)}")

        assert input_path.exists(), f"Input file {input_path} does not exist"

        if output_path.exists() and output_path.is_dir():
            output_path = output_path / input_path.with_suffix(self.output_suffix).name

        assert output_path.parent.exists(), f"Output folder {output_path.parent} does not exist"

        return input_path, output_path

    @abstractmethod
    def converter(
            self,
            input_path: str | Path,
            output_path: str | Path,
            /,
            **kwargs,
    ) -> None:
        raise NotImplementedError

    def folder_converter(
            self,
            input_folder: str | Path,
            output_folder: str | Path,
            **kwargs,
    ) -> None:
        if isinstance(input_folder, str):
            input_folder = Path(input_folder)
        elif isinstance(input_folder, Path):
            pass
        else:
            raise ValueError(f"input_folder must be a string or a Path object, not {type(input_folder)}")

        if isinstance(output_folder, str):
            output_folder = Path(output_folder)
        elif isinstance(output_folder, Path):
            pass
        else:
            raise ValueError(f"output_folder must be a string or a Path object, not {type(output_folder)}")

        assert input_folder.exists(), f"Input folder {input_folder} does not exist"

        if not output_folder.exists():
            output_folder.mkdir(parents=True)

        pbar = tqdm(list(input_folder.glob(self.file_glob)))
        pbar.set_description(f"Converting {input_folder} to {output_folder}")
        for input_path in pbar:
            output_path = output_folder / input_path.with_suffix(self.output_suffix).name
            pbar.set_postfix(
                input_path_name=input_path.name,
                # output_path_name=output_path.name,
            )
            self.converter(input_path, output_path, **kwargs)



# SPDX-FileCopyrightText: 2024-present Marceau <git@marceau-h.fr>
#
# SPDX-License-Identifier: AGPL-3.0-or-later
from camel2karam.converters.base_converter import PartsMapping
from camel2karam.converters.csv2connlu import Csv2Connlu
from camel2karam.converters.csvsent2csvtok import CsvSent2CsvTok
from camel2karam.converters.csvsent2onelettertag import CsvSent2OneLetterTag
from camel2karam.converters.csvtok2csvsent import CsvTok2CsvSent
from camel2karam.converters.jsonl2connlu import Jsonl2Connlu
from camel2karam.converters.jsonl2csv import Jsonl2Csv
from camel2karam.converters.jsonl2xml import Jsonl2Xml
from camel2karam.converters.tagset_convert import TagsetConvert


__all__ = [
    "PartsMapping",
    "Csv2Connlu",
    "CsvSent2CsvTok",
    "CsvSent2OneLetterTag",
    "CsvTok2CsvSent",
    "Jsonl2Connlu",
    "Jsonl2Csv",
    "Jsonl2Xml",
    "TagsetConvert",
]

from importlib import metadata
for dist in metadata.distributions():
    if dist.metadata["Name"] == "webanno_tsv":
        from camel2karam.converters.jsonl2webanno import Jsonl2WebAnno
        __all__.append("Jsonl2WebAnno")

        break
    else:
        print("""\
The `webanno_tsv` package is not installed. The `Jsonl2WebAnno` converter will not be available.
You can install it by running `pip install git+https://github.com/neuged/webanno_tsv.git`.""")



# SPDX-FileCopyrightText: 2024-present Marceau <git@marceau-h.fr>
#
# SPDX-License-Identifier: AGPL-3.0-or-later
from pathlib import Path
from typing import Optional

import polars as pl

from camel2karam.converters.base_converter import BaseConverter


class CsvSent2OneLetterTag(BaseConverter):
    corresp = {
        "PART": "P",
        "NOUN": "N",
        "PUNC": "Z",
        "VERB": "V",
        "DIGIT": "D",
        "NOUN_OR_PART": "N",
    }

    @classmethod
    def transform(cls, liste):
        return " | ".join([cls.corresp.get(tag.upper(), "X") for tag in liste.split(" | ")])

    def __init__(self):
        super().__init__(
            file_glob="*.csv",
            output_suffix=".csv",
        )

    def converter(
            self,
            input_path: str | Path,
            output_path: str | Path,
            /,
            word_col: str = "word",
            pos_col: str = "pos",
            pos2_col: Optional[str] = "xpos",
            **kwargs,
    ) -> None:
        input_path, output_path = self.converter_input_validation(input_path, output_path)

        df = pl.read_csv(input_path).fill_nan("").fill_null("")

        if pos2_col is None or pos_col not in df.schema:
            pos2_col = "empty_col"
            df = df.with_columns(
                pl.lit("").alias("empty_col")
            )

        df = df.select([word_col, pos_col, pos2_col])
        df = df.with_columns(
            pl.col(pos_col).map_elements(self.transform, return_dtype=pl.String).alias(pos_col),
            pl.col(pos2_col).map_elements(self.transform, return_dtype=pl.String).alias(pos2_col),
        )

        if pos2_col == "empty_col":
            df = df.select([word_col, pos_col])

        df.write_csv(output_path)

if __name__ == "__main__":
    cs2olt = CsvSent2OneLetterTag()

    cs2olt.folder_converter(
        "../../output/mled_msa_csv_sent",
        "../../output/mled_msa_csv_sent_olt",
        word_col="word",
        pos_col="pos",
        pos2_col=None
    )




# SPDX-FileCopyrightText: 2024-present Marceau <git@marceau-h.fr>
#
# SPDX-License-Identifier: AGPL-3.0-or-later
import json
from enum import StrEnum
from pathlib import Path
from typing import Optional

import polars as pl
from tqdm.auto import tqdm
from camel_tools.disambig.bert.unfactored import BERTUnfactoredDisambiguator
from camel_tools.disambig.mle import MLEDisambiguator
from camel_tools.tokenizers.word import simple_word_tokenize
# from camel_tools.tagger.default import DefaultTagger

class Disambiguator(StrEnum):
    MLED_MSA = "mled_msa"
    MLED_EGY = "mled_egy"
    BERT_MSA = "bert_msa"
    BERT_EGY = "bert_egy"
    BERT_GLF = "bert_glf"
    BERT_LEV = "bert_lev"


def mled_init(disambiguator: Optional[Disambiguator] = None):
    """apply camel-tools POS tagging according to tags listed in tags_list.json"""

    if disambiguator is None:
        disambiguator = Disambiguator.MLED_MSA

    disambiguators = {
        "mled_msa": [MLEDisambiguator.pretrained, {"model_name": "calima-msa-r13"}],
        "mled_egy": [MLEDisambiguator.pretrained, {"model_name": "calima-egy-r13"}],

        "bert_msa": [BERTUnfactoredDisambiguator.pretrained, {"model_name": "msa"}],
        "bert_egy": [BERTUnfactoredDisambiguator.pretrained, {"model_name": "egy"}],
        "bert_glf": [BERTUnfactoredDisambiguator.pretrained, {"model_name": "glf"}],
        "bert_lev": [BERTUnfactoredDisambiguator.pretrained, {"model_name": "lev"}],
    }

    try:
        mled = disambiguators[disambiguator]
        mled = mled[0](**mled[1])
    except KeyError:
        raise ValueError(f"disambiguator must be one of {list(disambiguators.keys())}")

    return mled


def tagger(
        input_path: str | Path,
        output_path: str | Path,
        mled: MLEDisambiguator,
        tags_list: list[str],
        debug=False
) -> None:
    if isinstance(input_path, str):
        input_path = Path(input_path)
    elif isinstance(input_path, Path):
        pass
    else:
        raise ValueError(f"input_path must be a string or a Path object, not {type(input_path)}")

    if isinstance(output_path, str):
        output_path = Path(output_path)
    elif isinstance(output_path, Path):
        pass
    else:
        raise ValueError(f"output_path must be a string or a Path object, not {type(output_path)}")

    assert input_path.exists(), f"Input file {input_path} does not exist"

    if output_path.exists() and output_path.is_dir():
        output_path = output_path / input_path.with_suffix(".jsonl").name

    assert output_path.parent.exists(), f"Output folder {output_path.parent} does not exist"

    with input_path.open('r', encoding='utf-8') as f:
        file = f.read()

    if debug:
        len_file = len(file)
        len_file_words = len(file.split())

    tokens = simple_word_tokenize(file)  # split_digits

    if debug:
        len_tokens = len(tokens)

    if isinstance(mled, BERTUnfactoredDisambiguator):
        batch_size = 10_000
        tokens_batches = [tokens[i:i + batch_size] for i in range(0, len(tokens), batch_size)]
        disambig = []
        for tokens_batch in tokens_batches:
            disambig.extend(mled.disambiguate(tokens_batch))


    else:
        disambig = mled.disambiguate(tokens)

    liste = [
        {
            "id": i,
            "word": tok,
            **{
                k: d.analyses[0].analysis[k]
                for k in tags_list
                if d.analyses
                and k in d.analyses[0].analysis
            }
        }
        for i, (tok, d) in enumerate(zip(tokens, disambig))
    ]

    if debug:
        len_liste = len(liste)

    liste = [
        {
            k: v
            if v not in {"", "na"}
            else v
            if k!="vox" and v!="u"
            else None
            for k, v in d.items()
        }
        for d in liste
    ]

    if debug:
        len_liste_na = len(liste)

    df = pl.DataFrame(liste)

    if debug:
        len_df = len(df)

        print(
            f"File: {input_path}\n"
            f"File length: {len_file}\n"
            f"File words: {len_file_words}\n"
            f"Tokens: {len_tokens}\n"
            f"Liste: {len_liste}\n"
            f"Liste na: {len_liste_na}\n"
            f"DataFrame: {len_df}\n"
        )

    df.write_ndjson(output_path)
    df.write_json(output_path.with_suffix(".json"))


def folder_tagger(
        input_folder: str | Path,
        output_folder: str | Path,
        mled: MLEDisambiguator,
        tags_list: list[str]
) -> None:
    if isinstance(input_folder, str):
        input_folder = Path(input_folder)
    elif isinstance(input_folder, Path):
        pass
    else:
        raise ValueError(f"input_folder must be a string or a Path object, not {type(input_folder)}")

    if isinstance(output_folder, str):
        output_folder = Path(output_folder)
    elif isinstance(output_folder, Path):
        pass
    else:
        raise ValueError(f"output_folder must be a string or a Path object, not {type(output_folder)}")

    assert input_folder.exists(), f"Input folder {input_folder} does not exist"

    output_folder.mkdir(parents=True, exist_ok=True)

    pbar = tqdm(sorted(input_folder.glob("*.txt"), key=lambda x: int(x.stem.strip().split()[-1])))
    for path in pbar:
        new_path = output_folder
        pbar.set_description(f"Processing {path}")

        tagger(path, new_path, mled, tags_list)


def main(
        input_folder: str | Path,
        output_folder: str | Path,
        disambiguator: Optional[Disambiguator] = None
):
    mled = mled_init(disambiguator)

    with open("../tags_list.json", "r") as f:
        tags_list = json.load(f)

    folder_tagger(input_folder, output_folder, mled, tags_list)


if __name__ == "__main__":
    for disamb in Disambiguator:
        print(disamb)
        main("../txts", "../output/" + disamb, disamb)




from argparse import ArgumentParser

from __about__ import __version__

def main():
    parser = ArgumentParser()
    parser.add_argument("-v", "--version", action="version", version=f"%(prog)s {__version__}")
    args = parser.parse_args()
    raise NotImplementedError("The cli is not implemented yet !")



# SPDX-FileCopyrightText: 2024-present Marceau <git@marceau-h.fr>
#
# SPDX-License-Identifier: AGPL-3.0-or-later
from pathlib import Path

import polars as pl

from camel2karam.converters.base_converter import BaseConverter

class Jsonl2Csv(BaseConverter):
    def __init__(self):
        self.fields_to_keep = ["id", "word", "pos"]

        super().__init__(
            file_glob="*.jsonl",
            output_suffix=".csv",
        )

    def converter(
            self,
            input_path: str | Path,
            output_path: str | Path,
            /,
            **kwargs,
    ) -> None:
        input_path, output_path = self.converter_input_validation(input_path, output_path)

        df = pl.read_ndjson(input_path)

        df = df.select(self.fields_to_keep)

        df.write_csv(output_path)


if __name__ == "__main__":
    j2c = Jsonl2Csv()

    j2c.folder_converter("../../output/mled_msa", "../../output/mled_msa_csv")
    j2c.folder_converter("../../output/mled_egy", "../../output/mled_egy_csv")
    j2c.folder_converter("../../output/bert_msa", "../../output/bert_msa_csv")
    j2c.folder_converter("../../output/bert_egy", "../../output/bert_egy_csv")
    j2c.folder_converter("../../output/bert_glf", "../../output/bert_glf_csv")
    j2c.folder_converter("../../output/bert_lev", "../../output/bert_lev_csv")

    j2c.folder_converter("../../output_karamed/mled_msa_karamed", "../../output_karamed/mled_msa_karamed_csv")
    j2c.folder_converter("../../output_karamed/mled_egy_karamed", "../../output_karamed/mled_egy_karamed_csv")
    j2c.folder_converter("../../output_karamed/bert_msa_karamed", "../../output_karamed/bert_msa_karamed_csv")
    j2c.folder_converter("../../output_karamed/bert_egy_karamed", "../../output_karamed/bert_egy_karamed_csv")
    j2c.folder_converter("../../output_karamed/bert_glf_karamed", "../../output_karamed/bert_glf_karamed_csv")
    j2c.folder_converter("../../output_karamed/bert_lev_karamed", "../../output_karamed/bert_lev_karamed_csv")



# SPDX-FileCopyrightText: 2024-present Marceau <git@marceau-h.fr>
#
# SPDX-License-Identifier: AGPL-3.0-or-later
from pathlib import Path

import polars as pl

from camel2karam.converters.base_converter import BaseConverter, PartsMapping


class Csv2Connlu(BaseConverter):
    parts_mapping = PartsMapping(
        FORM=0,
        UPOS=1,
        XPOS=2,
    )

    def __init__(self):
        super().__init__(
            file_glob="*.csv",
            output_suffix=".conllu",
        )

    def converter(
            self,
            input_path: str | Path,
            output_path: str | Path,
            /,
            word_col: str = "word",
            pos_col: str = "pos",
            pos2_col: str = "xpos",
            **kwargs,
    ) -> None:
        input_path, output_path = self.converter_input_validation(input_path, output_path)

        df = pl.read_csv(input_path).fill_nan("_").fill_null("_")

        if pos2_col not in df.columns or pos2_col is None:
            pos2_col = pl.Series("pos2_col", ["_"] * df.shape[0])
        else:
            pos2_col = df[pos2_col]

        sents = self.toks_to_sents(df[word_col], df[pos_col], pos2_col)

        self.write_connlu(output_path, sents, self.parts_mapping)


if __name__ == "__main__":
    c2c = Csv2Connlu()

    c2c.folder_converter(
        "../../output/mled_msa_csv",
        "../../output/mled_msa_conllu_from_csv",
        word_col="word",
        pos_col="pos",
        pos2_col=None
    )



# SPDX-FileCopyrightText: 2024-present Marceau <git@marceau-h.fr>
#
# SPDX-License-Identifier: AGPL-3.0-or-later
from pathlib import Path
from typing import Optional

import polars as pl

from camel2karam.converters.base_converter import BaseConverter


class CsvSent2CsvTok(BaseConverter):
    def __init__(self):
        super().__init__(
            file_glob="*.csv",
            output_suffix=".csv",
        )

    def converter(
            self,
            input_path: str | Path,
            output_path: str | Path,
            /,
            word_col: str = "word",
            pos_col: str = "pos",
            pos2_col: Optional[str] = "xpos",
            **kwargs,
    ) -> None:
        self.converter_input_validation(input_path, output_path)

        if output_path.exists() and output_path.is_dir():
            output_path = output_path / input_path.with_suffix(".conllu").name

        df = pl.read_csv(input_path).fill_nan("").fill_null("")

        if pos2_col is None or pos_col not in df.schema:
            pos2_col = "pos2"
            words, poss, pos2s = [], [], None

            df = df.with_columns(
                pl.lit("").alias("pos2")
            )
        else:
            words, poss, pos2s = [], [], []

        df = df.select([word_col, pos_col, pos2_col])


        for row in df.iter_rows():
            word, pos, pos2 = row
            word, pos, pos2 = word.split(" | "), pos.split(" | "), pos2.split(" | ")

            assert len(word) == len(pos)

            words.extend(word)
            poss.extend(pos)

            if pos2s is not None:
                pos2s.extend(pos2)

        if pos2s is None:
            new_df = pl.DataFrame({
                word_col: words,
                pos_col: poss
            })
        else:
            new_df = pl.DataFrame({
                word_col: words,
                pos_col: poss,
                pos2_col: pos2s
            })


        new_df.write_csv(output_path)

if __name__ == "__main__":
    cs2ct = CsvSent2CsvTok()

    cs2ct.folder_converter(
        "../../output/mled_msa_csv_sent",
        "../../output/mled_msa_csv_sent_tok",
        word_col="word",
        pos_col="pos",
        pos2_col=None
    )




# SPDX-FileCopyrightText: 2024-present Marceau <git@marceau-h.fr>
#
# SPDX-License-Identifier: AGPL-3.0-or-later
from pathlib import Path

import polars as pl

from camel2karam.converters.base_converter import BaseConverter, PartsMapping

class Jsonl2Connlu(BaseConverter):
    parts_mapping = PartsMapping(
        FORM=0,
        LEMMA=2,
        UPOS=1,
        XPOS=3,
    )
    def __init__(self):
        super().__init__(
            file_glob="*.jsonl",
            output_suffix=".conllu",
        )

    def from_line_to_xpos(self, serie):
        if not any(serie[k] for k in ["asp", "per", "gen", "num", "stt", "cas", "vox", "mod"]):
            pass
            return serie['pos'].upper()
        pos, asp, per, gen, num, stt, cas, vox, mod = [e if e != "_" else '' for e in serie.values()]
        return f"{pos}.{asp}{per}{gen}{num}.{stt}{cas}{vox}{mod}".upper().strip(".")

    def converter(
            self,
            input_path: str | Path,
            output_path: str | Path,
            /,
            **kwargs,
    ) -> None:
        input_path, output_path = self.converter_input_validation(input_path, output_path)

        df = pl.read_ndjson(input_path).fill_nan("_").fill_null("_")

        df = df.with_columns(
            pl.col("vox").str.replace("u", "_", literal=True)
        )

        df = df.with_columns(
                 pl.struct(
                        [
                            "pos",
                            "asp",
                            "per",
                            "gen",
                            "num",
                            "stt",
                            "cas",
                            "vox",
                            "mod",
                        ]
                    ).map_elements(
                self.from_line_to_xpos,
                return_dtype=pl.Utf8,
            ).alias("xpos")
        )

        if "atbtok" not in df.columns:
            df = df.with_columns(pl.lit("_").alias("atbtok"))

        sents = self.toks_to_sents(df['word'], df['pos'], df['atbtok'], df['xpos'])

        self.write_connlu(output_path, sents, self.parts_mapping)


if __name__ == "__main__":
    j2c = Jsonl2Connlu()

    j2c.folder_converter("../../output/mled_msa", "../../output/mled_msa_conllu")
    j2c.folder_converter("../../output/mled_egy", "../../output/mled_egy_conllu")
    j2c.folder_converter("../../output/bert_msa", "../../output/bert_msa_conllu")
    j2c.folder_converter("../../output/bert_egy", "../../output/bert_egy_conllu")
    j2c.folder_converter("../../output/bert_glf", "../../output/bert_glf_conllu")
    j2c.folder_converter("../../output/bert_lev", "../../output/bert_lev_conllu")

    j2c.folder_converter("../../output_karamed/mled_msa_karamed", "../../output_karamed/mled_msa_karamed_conllu")
    j2c.folder_converter("../../output_karamed/mled_egy_karamed", "../../output_karamed/mled_egy_karamed_conllu")
    j2c.folder_converter("../../output_karamed/bert_msa_karamed", "../../output_karamed/bert_msa_karamed_conllu")
    j2c.folder_converter("../../output_karamed/bert_egy_karamed", "../../output_karamed/bert_egy_karamed_conllu")
    j2c.folder_converter("../../output_karamed/bert_glf_karamed", "../../output_karamed/bert_glf_karamed_conllu")
    j2c.folder_converter("../../output_karamed/bert_lev_karamed", "../../output_karamed/bert_lev_karamed_conllu")
